/* Data Visualization - 2D Matrix (Processing 3)
 Based on Rodger's 2D Matrix Demo  
 
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::::::: How does Dead Rock Star Stay Alive :::::::::::::::::::::::
::::::::::::::::::::::::::::::::::::::::::::code: Jing Yan :::::::
::::::::::::::::::::theuniqueeye@gmail.com ::::::::::::::::::::::*/

// :: Interaction KEY 
// :: press R / r to show a ruler
// :: press C / c to apply the color scheme and do normalization

// Layout variables    
float constantVerticalMargin = 15;

// 2D matrix table related variables
Table table1, table2;
int numRows, numColumns;
float[][] dataMatrix;
float maxValue, minValue, maxBookValue;

// width and height for each cell and title
float cellWidth1, cellWidth2, cellHeight;  
float TitleWidth, TitleHeight;

boolean colorApplied = false;
boolean normalization = false;
boolean drawRuler = false;

// font for each text
PFont titleFont, subtitleFont, labelFont, noteFont;
PImage titleImage, labelImage1, labelImage2;


// Mouse data
boolean cdOrBook;
int mouseData;

// ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setup() {
  // create a vertical interface
  size(560, 960); 

  // Make the screen resizable.                     
  surface.setResizable(true);

  // Load checkout times for both CD and Book
  table1 = loadTable("CD_CheckoutByMonth.csv", "header");
  table2 = loadTable("BK_CheckoutByMonth.csv", "header");

  titleFont = loadFont("KohinoorDevanagari-Bold-13.vlw");
  subtitleFont = loadFont("KohinoorDevanagari-Medium-10.vlw");
  labelFont = loadFont("OratorStd-8.vlw");
  noteFont = loadFont("KohinoorDevanagari-Demi-7.vlw");

  titleImage = loadImage("title-04.png");
  labelImage1 = loadImage("label-05.png");
  labelImage2 = loadImage("label-06.png");

  cdOrBook = false;

  // Row: 2 types for book and CD
  // Column: every month from 2005/8 to 2015/1
  numColumns = 2; 
  numRows = 114; // 5+9*12+1 

  dataMatrix = new float[numRows][numColumns];
  println("Rows: " + numRows + " Columns: " + numColumns);

  // The following for loop is used to load data into dataMatrix.
  // It's important to keep the timeline evenly distribute.

  for (int i=0; i<numRows; i++) {

    // turn the variable into month-format
    // same as the month-format in CD.csv data
    String month= str(2000+(i+8)/12+5)+"-"+nf((i+8)%12+1, 2, 0);

    // load CD's data into the 1st column
    TableRow row = table1.findRow(month, 0);
    if (row!=null) {
      dataMatrix[i][0] = row.getInt(1);
    } else {
      dataMatrix[i][0] = 0;
    }

    // turn the variable into month-format
    // same as the month-format in Book.csv data
    month= str(2000+(i+8)/12+5)+"/"+nf((i+8)%12+1, 2, 0);

    // load book's data into the 2rd column
    row = table2.findRow(month, 0);
    if (row!=null) {
      dataMatrix[i][1] = row.getInt(1);
    } else {
      dataMatrix[i][1] = 0;
    }
  }

  // get the maxValue, minValue 
  maxValue = max2D(dataMatrix);
  minValue = min2D(dataMatrix);
  println("Max Value is " + maxValue);
  println("Min Value is " + minValue);

  pixelDensity(displayDensity());
  smooth();
  noStroke();
}

// ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void draw() {
  //noStroke(); 
  background(255);
  TitleWidth = 150;
  TitleHeight = 50;

  // the height of each cell is the same
  cellHeight = (height - constantVerticalMargin*2 - TitleHeight)/(numRows);

  // Draw 2D Matrix!!
  noStroke(); 
  rectMode(CORNER);

  // Break the 2D matrix into 2 parts 
  // 1st part
  for (int i=0; i<65; i++) {

    // the width of each cell is related to the data value
    cellWidth1 = map(dataMatrix[i][0], minValue, maxValue, 0, 200);
    cellWidth2 = map(dataMatrix[i][1], minValue, maxValue, 0, 200); 

    // draw cells along the hypotenuse of "lightening"
    float x = (2-float(i)/76)*width/3;
    float y = constantVerticalMargin + cellHeight * i; 
    float gap = 5;

    fill(128);

    rect(x-cellWidth1-gap, y, cellWidth1, 0.3*cellHeight);
    rect(x+gap, y, cellWidth2, 0.3*cellHeight);

    // interaction with mouse position 
    // when the mouse falls in the range of each data, 
    // show the number of the data on left down corner of screen
    // when the mouse falls in the CD or book part of data, darken the label respectively
    
    boolean isMouseXIn = mouseX < (x-cellWidth1-gap+cellWidth1);
    boolean isMouseYIn = mouseY > y && mouseY < y + 0.3*cellHeight;
    if (isMouseXIn && isMouseYIn) {
      mouseData = int(dataMatrix[i][0]);
      cdOrBook = false;
    }

    isMouseXIn = mouseX > x+gap ;
    isMouseYIn = mouseY > y && mouseY < y + 0.3*cellHeight;
    if (isMouseXIn && isMouseYIn) {
      mouseData = int(dataMatrix[i][1]);
      cdOrBook = true;
    }
  }

  // 2nd part
  for (int i=65; i<114; i++) {

    cellWidth1 = map(dataMatrix[i][0], minValue, maxValue, 0, 200);
    cellWidth2 = map(dataMatrix[i][1], minValue, maxValue, 0, 200); 

    float x = (2-float(i)/76)*width/3 + TitleWidth;
    float y = constantVerticalMargin + cellHeight * i+TitleHeight; 
    float gap = 5;

    fill(128);

    rect(x-cellWidth1-gap, y, cellWidth1, 0.3*cellHeight);
    rect(x+gap, y, cellWidth2, 0.3*cellHeight);  
    
    // interaction with mouse position 
    // when the mouse falls in the range of each data, 
    // show the number of the data on left down corner of screen
    // when the mouse falls in the CD or book part of data, darken the label respectively
    
    boolean isMouseXIn = mouseX < (x-cellWidth1-gap+cellWidth1);
    boolean isMouseYIn = mouseY > y && mouseY < y + 0.3*cellHeight;
    if (isMouseXIn && isMouseYIn) {
      mouseData = int(dataMatrix[i][0]);
      cdOrBook = false;
    }

    isMouseXIn = mouseX > x+gap;
    isMouseYIn = mouseY > y && mouseY < y + 0.3*cellHeight;
    if (isMouseXIn && isMouseYIn) {
      mouseData = int(dataMatrix[i][1]);
      cdOrBook = true;
    }
  }

  // apply color and normalization
  if (colorApplied == true) colorScheme();
  
  // draw a timeline
  if (drawRuler == true) drawRuler();
  
  // draw title and labels and note
  drawText();
}


// ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::